@@ -21,7 +21,7 @@ compensates for some of the idiosyncrasies of ODBC.
 
 =cut
 
-=head2 CaseSensitive
+=head2 case_sensitive
 
 Returns a false value.
 
@@ -32,6 +32,29 @@ sub case_sensitive {
     return (undef);
 }
 
+=head2 build_dsn
+
+=cut
+
+sub build_dsn {
+    my $self = shift;
+    my %args = (
+        driver   => undef,
+        database => undef,
+        host     => undef,
+        port     => undef,
+        @_
+    );
+
+    $args{dbname} ||= delete $args{database};
+
+    my $dsn = "dbi:$args{driver}:$args{dbname}";
+    $dsn .= ";host=$args{'host'}" if $args{'host'};
+    $dsn .= ";port=$args{'port'}" if $args{'port'};
+
+    $self->{'dsn'} = $dsn;
+}
+
 =head2 apply_limits
 
 =cut
@@ -47,21 +70,24 @@ sub apply_limits {
     $$statementref =~ s/SELECT\b/SELECT $limit_clause/;
 }
 
-=head2 DistinctQuery
+=head2 distinct_query
 
 =cut
 
 sub distinct_query {
     my $self         = shift;
     my $statementref = shift;
-
-    my $sb = shift;
+    my $collection   = shift;
 
     $$statementref = "SELECT main.* FROM $$statementref";
-    $$statementref .= $sb->_group_clause;
-    $$statementref .= $sb->_order_clause;
+    $$statementref .= $collection->_group_clause;
+    $$statementref .= $collection->_order_clause;
 }
 
+=head2 encoding
+
+=cut
+
 sub encoding {
 }
 
@@ -71,7 +97,7 @@ __END__
 
 =head1 AUTHOR
 
-Autrijus Tang
+Audrey Tang C<cpan@audreyt.org>
 
 =head1 SEE ALSO
 
